#include <signal.h>
#include <io.h>
#include "usart.h"

static void cpuInit(void)
{
  register int delay;

  WDTCTL = WDTPW + WDTHOLD;             // stop Watchdog
  BCSCTL1 |= XTS + DIVA1 + XT2OFF;      // LFXT1 = HF XTAL
                                        // ACLK = LFXT1/4
                                        // XT2 Osc is OFF

  // setup ports while waiting for (LF)XT1 to come up
  // setup port 1 from values in config.h
  P1SEL = P1SEL_INIT;
  P1OUT = P1OUT_INIT;
  P1DIR = P1DIR_INIT;

  // setup port 2 from values in config.h
  P2SEL = P2SEL_INIT;
  P2OUT = P2OUT_INIT;
  P2DIR = P2DIR_INIT;

  // setup port 3 from values in config.h
  P3SEL = P3SEL_INIT;
  P3OUT = P3OUT_INIT;
  P3DIR = P3DIR_INIT;

  // setup port 4 from values in config.h
  P4SEL = P4SEL_INIT;
  P4OUT = P4OUT_INIT;
  P4DIR = P4DIR_INIT;

  // setup port 5 from values in config.h
  P5SEL = P5SEL_INIT;
  P5OUT = P5OUT_INIT;
  P5DIR = P5DIR_INIT;

  // setup port 6 from values in config.h
  P6SEL = P6SEL_INIT;
  P6OUT = P6OUT_INIT;
  P6DIR = P6DIR_INIT;

  // wait for (LF)XT1 oscillator to come up
  // clear OSC fault flag & set delay count
  // exit loop when no fault for delay count
  for (IFG1 &= ~OFIFG, delay = 255; delay; )
    {
    if (IFG1 & OFIFG)                   // check OSC fault flag
      {
      // it's set
      IFG1 &= ~OFIFG;                   // clear OSC fault flag
      delay = 255;                      // reset delay count
      }
    else
      --delay;                          // clear, decrement delay
    }

  BCSCTL2 |= SELM1 + SELM0 +            // MCLK = LFXT1/1
             DIVS1 + DIVS0;             // SMCLK = DCOCLK/8

#ifdef RELEASE
  // LAST step is re-enable the watchdog timer
  // - use tSMCLK * 2e15 (approx 350mSec)
  WDTCTL = WDTPW + WDTCNTCL;
#endif
}


int main(void)
{
  int ch;

  cpuInit();

  usart0Init(USART0_BAUD_DIV(9600), USART0_BAUD_MOD(9600), USART_8N1);
//  usart1Init(USART0_BAUD_DIV(9600), USART0_BAUD_MOD(9600), USART_8N1);
  eint();
  usart0Puts("\n\rHello World\n\r");

  for (;;)
    {
    if ((ch = usart0Getch()) >= 0)
      usart0Putch(ch);

//    if ((ch = usart1Getch()) >= 0)
//      usart1Putch(ch);

    WDOG();
    }

  return 0;                             // should NEVER get here
}
