#include "hardware.h"
#include "gfxcore.h"
#include "gfxbasic.h"

//this one is used in drawing funcs. it has nothing to do with
//what the display realy displays. this makes it possible to
//do double buffering.
unsigned short gfxbase;

//write a character at the current cursor pos
void gfxPutc(char c) {
    if( c >= 32 ) {
        dput(c - 32);
        cput(0xC0);
    }
}

//initialize LCD memory and display modes
void gfx_init(void) {
    gfxbase = G_BASE;                           //set base pointer
    cput16(gfxbase, 0x42);                      //set graphics memory to address G_BASE
    cput16(GFX_BYTES_PER_ROW, 0x43);            //n bytes per graphics line
    cput16(T_BASE, 0x40);                       //text memory at address T_BASE
    cput16(TXT_BYTES_PER_ROW, 0x41);            //n bytes per text line
    cput(0x80);                                 //mode set: Graphics OR Text, ROM CGen
    clear_lcd_text();                           //clear text
    clear_lcd_gfx();                            //and graphics
    cput(0xA7);                                 //cursor is 8 lines high
    cput16(0, 0x21);                            //put cursor at (x,y) location
    cput(0x9C);                                 //graphics & text ON, cursor blinking
                                                //(for cursor to be visible, need to set up position)
}

void set_cursor(unsigned char x, unsigned char y) {
    dput(x);
    dput(y);
    cput(0x21);
}

//set the text cursor
void set_pointer(unsigned char x, unsigned char y) {
    cput16(T_BASE + ((unsigned short)y * TXT_BYTES_PER_ROW) + x, 0x24);
    }

void clear_lcd_text(void) {
    unsigned int i;
    cput16(T_BASE, 0x24);                       //set pointer on start of text mem
    for (i = 0; i < TXT_LINES*TXT_BYTES_PER_ROW; i++) { //for each character
        dput(0);                                //put a zero
        cput(0xC0);                             //write
    }
}

//set single pixel
void lcd_setpixel(unsigned char x, unsigned char y) {
    unsigned int address;       //memory address of byte containing pixel to write
    address = gfxbase + (y*GFX_BYTES_PER_ROW)  + (x/8);
    cput16(address, 0x24);      //set LCD addr. pointer
    
    cput(0xf8 | (7-(x&7)) );    //set bit-within-byte command
}

//this is a very primitive implemented filled rect...
void fill_rectangle(unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2) {
    unsigned short temp = x1;
    for (; y1 <= y2; y1++) {
        for (x1 = temp; x1 <= x2; x1++) {
            lcd_setpixel(x1, y1);
        }
    }
}

//draw the borders of a rectangle
void rectangle(unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2) {
    unsigned short temp;
    for (temp = x1; temp <= x2; temp++) {
        lcd_setpixel(temp, y1);
        lcd_setpixel(temp, y2);
    }
    for (temp = y1; temp <= y2; temp++) {
        lcd_setpixel(x1, temp);
        lcd_setpixel(x2, temp);
    }
}

//put all zeros in gfx memory to clear it
void clear_lcd_gfx(void) {
    unsigned int i;
    cput16(gfxbase, 0x24);              //start of gfx mem
    cput(0xb0);                         //start auto write mode
    for (i = 0; i < GFX_HEIGHT*GFX_BYTES_PER_ROW; i++) {
        autowr(0);
    }
    cput(0xb2);                         //reset auto mode
}

