#include "hardware.h"

.text

//wait until display is ready
.global s_wait
        .type   s_wait,@function
s_wait: bis.b   #0x0f, &LCDCMDOUT
        clr.b   &LCDDATADIR             //make 8-bit parallel port an input
        bis.b   #CD, &LCDCMDOUT         //bring LCD C/D line high (read status byte)
        bic.b   #RD, &LCDCMDOUT         //bring LCD /RD line low (read active)
swl0:   bic.b   #CE, &LCDCMDOUT         //bring LCD /CE line low (chip-enable active)
        mov.b   &LCDDATAIN, r12         //read LCD status byte
        bis.b   #CE, &LCDCMDOUT         //bring LCD /CE line high, disabling it
        bit.b   #0x01, r12              //STA0
        jz      swl0
        bit.b   #0x02, r12              //STA1
        jz      swl0
        bis.b   #RD, &LCDCMDOUT         //deactivate LCD read mode
        mov.b   #0xff, &LCDDATADIR      //make 8-bit parallel port an output
        ret

//wait until auto data write possible
.global waitAW
        .type   waitAW,@function
waitAW: bis.b   #0x0f, &LCDCMDOUT
        clr.b   &LCDDATADIR             //make 8-bit parallel port an input
        bis.b   #CD, &LCDCMDOUT         //bring LCD C/D line high (read status byte)
        bic.b   #RD, &LCDCMDOUT         //bring LCD /RD line low (read active)
sw2l0:  bic.b   #CE, &LCDCMDOUT         //bring LCD /CE line low (chip-enable active)
        mov.b   &LCDDATAIN, r12         //read LCD status byte
        bis.b   #CE, &LCDCMDOUT         //bring LCD /CE line high, disabling it
        bit.b   #0x08, r12              //STA3
        jz      sw2l0
        bis.b   #RD, &LCDCMDOUT         //deactivate LCD read mode
        mov.b   #0xff, &LCDDATADIR      //make 8-bit parallel port an output
        ret


//write a 16 bit value / two bytes to the display data stack
.global dput16
        .type   dput16,@function
dput16: call    #s_wait                 //wait until display ready
        bic.b   #CD|WR, &LCDCMDOUT      //activate LCD's write mode
        mov.b   r15, &LCDDATAOUT        //write value to data port
        bic.b   #CE, &LCDCMDOUT         //pulse enable LOW > 80 ns
        bis.b   #CE|WR, &LCDCMDOUT      //restore Write mode to inactive
        swpb    r15                     //first value written, swap an go on for the second...
                                        //dput MUST follow!
//write a eight bit valuet to the display data stack
.global dput
        .type   dput,@function
dput:   call    #s_wait                 //wait until display ready
        bic.b   #CD|WR, &LCDCMDOUT      //activate LCD's write mode
        mov.b   r15, &LCDDATAOUT        //write value to data port
        bic.b   #CE, &LCDCMDOUT         //pulse enable LOW > 80 ns 
        bis.b   #CE|WR|CD, &LCDCMDOUT   //restore Write mode to inactive
        ret

//loop function for autowrite mode
.global autowr
        .type   autowr,@function
autowr: call    #waitAW                 //wait until display ready
        bic.b   #CD|WR, &LCDCMDOUT      //activate LCD's write mode
        mov.b   r15, &LCDDATAOUT        //write value to data port
        bic.b   #CE, &LCDCMDOUT         //pulse enable LOW > 80 ns
        bis.b   #CE|WR|CD, &LCDCMDOUT   //restore Write mode to inactive
        ret

// get data byte from LCD module
.global dget
        .type   dget,@function
dget:   call    #s_wait                 //wait until display ready
        clr.b   &LCDDATADIR             //make port input
        bis.b   #WR, &LCDCMDOUT         //not write..
        bic.b   #CD|RD, &LCDCMDOUT      //but enable read mode
        bic.b   #CE, &LCDCMDOUT         //chip enable
        mov.b   &LCDDATAIN, r15         //read data from LCD
        bis.b   #CE|RD, &LCDCMDOUT      //disable read, go idle
        mov.b   #0xff, &LCDDATADIR      //swich back to outputs, as default
        ret

//execute display function with 16 bit argument
.global cput16
        .type   cput16,@function
cput16: call    #dput16                 //put the arguments on the display data stack
        mov.b   r14, r15                //move command to the right register for cput
                                        //cput MUST follow...
//execute display function
.global cput
        .type   cput,@function
cput:   call    #s_wait                 //wait until display ready
        mov.b   r15, &LCDDATAOUT        //output data
        bis.b   #CD|RD, &LCDCMDOUT      //make sure LCD read mode is off
        bic.b   #WR, &LCDCMDOUT         //activate LCD write mode
        bic.b   #CE, &LCDCMDOUT         //pulse ChipEnable LOW, > 80 ns, enables LCD I/O
        bis.b   #CE|WR, &LCDCMDOUT      //disable LCD I/O, deactivate write mode
        ret
