#ifndef HARDWARE_H
#define HARDWARE_H

#include <io.h>
#include <signal.h>
#include <iomacros.h>

//if you're low on flash, undef this one:
#define USE_TITLE

//ports
#define P1OUT_INIT      0               //Init Output data of port1
#define P2OUT_INIT      0               //Init Output data of port2
#define P3OUT_INIT      0               //Init Output data of port3
#define P4OUT_INIT      0               //Init Output data of port4
#define P5OUT_INIT      0               //Init Output data of port5
#define P6OUT_INIT      0               //Init Output data of port6

#define P1DIR_INIT      0xff            //Init of Port1 Data-Direction Reg (Out=1 / Inp=0)
#define P2DIR_INIT      0xff            //Init of Port2 Data-Direction Reg (Out=1 / Inp=0)
#define P3DIR_INIT      ~(BIT5|BIT3)    //Init of Port3 Data-Direction Reg: UART0 RX,CTS
#define P4DIR_INIT      0xff            //Init of Port4 Data-Direction Reg
#define P5DIR_INIT      0xff            //Init of Port5 Data-Direction Reg
#define P6DIR_INIT      0               //Init of Port6 Data-Direction Reg

#define P1SEL_INIT      0               //P1-Modules:
#define P2SEL_INIT      0               //P2-Modules:
#define P3SEL_INIT      0               //P3-Modules:
#define P4SEL_INIT      0               //P4-Modules:
#define P5SEL_INIT      0               //P5-Modules:
#define P6SEL_INIT      0               //P6-Modules:

#define P1IE_INIT       0               //P1-Interrupts:
#define P2IE_INIT       0               //P2-Interrupts:
#define P1IES_INIT      0               //P1-Interrupts:
#define P2IES_INIT      0               //P2-Interrupts:



//gfx LCD connections
//~ #define LCDDATADIR      P4DIR
//~ #define LCDDATAIN       P4IN
//~ #define LCDDATAOUT      P4OUT
//~ #define LCDCMDOUT       P5OUT
#define LCDDATADIR      P3DIR
#define LCDDATAIN       P3IN
#define LCDDATAOUT      P3OUT
#define LCDCMDOUT       P2OUT
//bit names
#define WR              0x01            //Px.0
#define RD              0x02            //Px.1
#define CE              0x04            //Px.2
#define CD              0x08            //Px.3
#define RESET           0x10            //Px.4
#define FONTSEL         0x20            //Px.5

#define G_BASE          0x0200          //base address of graphics memory
#define G_BASE_1        (2560+G_BASE)   //base address of graphics memory
#define T_BASE          0x0000          //base address of text memory
#define TXT_BYTES_PER_ROW       40      //how many bytes per row on screen
#define TXT_LINES               8       //number of lines in text mode
#define GFX_BYTES_PER_ROW       32      //how many bytes per row on screen
#define GFX_WIDTH              240      //number of columns in pixels
#define GFX_HEIGHT              64      //number of rows in pixels


#define BCSCTL1_INIT    XT2OFF|RSEL2|RSEL1|RSEL0   //just make it fast
#define BCSCTL2_INIT    0

#endif  //HARDWARE_H
