#include "hardware.h"
#include "gfxcore.h"
#include "gfxbasic.h"
#include <stdlib.h>
#include <stdio.h>

//redirect stdio looking funcs to LCD
#define printf(args...) uprintf(gfxPutc, args)
#define putc            gfxPutc

extern const char title[];

//the known busy wait type wait function
void delay(unsigned int d) {
    int i;
    for (i = 0; i<d; i++) {
        nop();
        nop();
    }
}

//draw a small ball at specified coordinate
void drawball(unsigned short x, unsigned short y) {
    lcd_setpixel(x+1,y+0);
    lcd_setpixel(x+2,y+0);
    lcd_setpixel(x+0,y+1);
    lcd_setpixel(x+1,y+1);
    lcd_setpixel(x+2,y+1);
    lcd_setpixel(x+3,y+1);
    lcd_setpixel(x+0,y+2);
    lcd_setpixel(x+1,y+2);
    lcd_setpixel(x+2,y+2);
    lcd_setpixel(x+3,y+2);
    lcd_setpixel(x+1,y+3);
    lcd_setpixel(x+2,y+3);
}


//do the fun stuff...
int main(void) {
    unsigned char i, j, l, m, frame = 0;
    unsigned short x, y, dx, dy;

    WDTCTL = WDTPW + WDTHOLD;           //stop watchdog timer

    BCSCTL1 = BCSCTL1_INIT;             //setup clock system
    BCSCTL2 = BCSCTL2_INIT;

    P1OUT = P1OUT_INIT;                 //init ports
    P2OUT = P2OUT_INIT;
    P3OUT = P3OUT_INIT;

    P1SEL = P1SEL_INIT;                 //port mode
    P2SEL = P2SEL_INIT;
    P3SEL = P3SEL_INIT;

    P1IES = P1IES_INIT;                 //port interrupts
    P2IES = P2IES_INIT;
    P1IE  = P1IE_INIT;
    P2IE  = P2IE_INIT;
    
    P1DIR = P1DIR_INIT;                 //and finaly the direction
    P2DIR = P2DIR_INIT;
    P3DIR = P3DIR_INIT;
    
    #ifdef __msp430_have_port4          //only for the bigger parts...
    P4OUT = P4OUT_INIT;
    P5OUT = P5OUT_INIT;
    P6OUT = P6OUT_INIT;
    
    P4SEL = P4SEL_INIT;
    P5SEL = P5SEL_INIT;
    P6SEL = P6SEL_INIT;

    P4DIR = P4DIR_INIT;
    P5DIR = P5DIR_INIT;
    P6DIR = P6DIR_INIT;
    #endif //__msp430_have_port4

    LCDCMDOUT = 0xFF;                   //indle control lines
    
    LCDCMDOUT &= ~RESET;                //do a display reset
    delay(500);                         //hold the pin down for some time
    LCDCMDOUT |= RESET;                 //ok
    
    gfx_init();
    LCDCMDOUT &= ~FONTSEL;              //select 8 pixels with font
    
    gfxbase = G_BASE;                   //set base address
    
#ifdef USE_TITLE
    for (y = 0; y < 64; y++) {          //copy the pixels...
        for (x = 0; x < 240; x++) {     //the size here is hardcoded as the image is fixed too
            if (!title[x+y*240]) lcd_setpixel(x,y);
        }
    }
    
    for( i = 0; i<50; i++ ){
        delay(60000);                   //keep this display for a moment
    }
#endif //USE_TITLE
    
    set_pointer(2, 3);                  //move text cursor
    printf("http://mspgcc.sf.net");
    cput(0x81);                         //mode set: Graphics EXOR Text, ROM CGen

    x = y = 10;                         //set ball start-coordinates
    dx = dy = 2;                        //initial movement
    eint();                             //enable interrupts
    
    while (1) {                         //the usual never ending main loop
        i = 0;
        j = GFX_WIDTH-1;
        l = 0;
        m = GFX_HEIGHT-9;
        frame = 0;
        
        for (; m>=l ; m--, l++, i++, j--, frame^=-1) {          //thats an ugly for....
            P1OUT = 1<<(l&7);           //no that's of no use - just that we see that it runs
            
            if ((x<=0) || (x >= (GFX_WIDTH-3))) dx = -dx;       //reflect ball at the borders
            if ((y<=0) || (y >= (GFX_HEIGHT-3)))  dy = -dy;     //reflect ball at the borders
            x += dx;                                            //move the ball
            y += dy;                                            //move the ball
            
            if (!frame) {                                       //swap the frames!
                gfxbase = G_BASE;                       //select frame 0 for drawing
                clear_lcd_gfx(); //clear_lcd_grafic_fast(0);
    
                rectangle(i, l, j, m);
                drawball(x,y);
    
                set_pointer(1, TXT_LINES-1);
                printf("%02x %02x ", i, j);
                printf("%02x %02x ", l, m);
                printf("Screen: 0");
    
                cput16((unsigned short)G_BASE, 0x42);           //set graphics memory to address G_BASE
            } else {
                gfxbase = G_BASE_1;
                clear_lcd_gfx(); //clear_lcd_grafic_fast(1);
    
                rectangle(i, l, j, m);
                drawball(x,y);
    
                set_pointer(1, TXT_LINES-1);
                printf("%02x %02x ", i, j);
                printf("%02x %02x ", l, m);
                printf("Screen: 1");
    
                cput16((unsigned short)G_BASE_1, 0x42);         //set graphics memory to address G_BASE_1
            }
        }
    }
}
