/******************************************************************************
 *
 * $Workfile: config.h $
 *
 * This module provides information about the project configuration
 *
 * $History: config.h $
 *
 *****************************************************************************/
#ifndef INC_CONFIG_H
#define INC_CONFIG_H

#include <io.h>
#include <sys/inttypes.h>

// Handy Macros
// service the watchdog timer
#define WDOG()              (WDTCTL = (WDTCTL & 0xFF) + WDTPW + WDTCNTCL)

// If we don't use a tasker, define swap() here
//#define swap()              WDOG()

// Oscillator Frequencies
#define LFXT1CLK            (6.78e6)
#define XT2CLK
#define DCOCLK              (750e3)     // nominal Rsel=4, DCO=3, MOD=0, DCOR=0

// Clock Frequencies
#define ACLK                (LFXT1CLK / 4)
#define MCLK                LFXT1CLK
#define SMCLK               (DCOCLK / 8)

// System Data Rates
#define HOST_BAUD           (9600ul)

// Port Usage Definitions & Access Macros
#ifdef __msp430_have_port0
// Port 0 Bits
// (not used)

// Port 0 Inits
#define P0SEL_INIT          (0x00)      // all pins are port pins
#define P0OUT_INIT          (0x00)      // start with all output pins LOW
#define P0DIR_INIT          (0xFF)      // all pins are outputs

// Port 0 Access Macros
// (not used)
#endif

#ifdef __msp430_have_port1
// Port 1 Bits
// (not used)

// Port 1 Inits
#define P1SEL_INIT          (0x00)      // all pins are port pins
#define P1OUT_INIT          (0x00)      // start with all output pins LOW
#define P1DIR_INIT          (0xFF)      // all pins are outputs

// Port 1 Access Macros
// (not used)
#endif

#ifdef __msp430_have_port2
// Port 2 Bits
// (not used)

// Port 2 Inits
#define P2SEL_INIT          (0x00)      // all pins are port pins
#define P2OUT_INIT          (0x00)      // start with all output pins LOW
#define P2DIR_INIT          (0xFF)      // all pins are outputs

// Port 2 Access Macros
// (not used)
#endif

#ifdef __msp430_have_port3
// Port 3 Bits
#define HOST_TX_BIT         BV(4)       // SCI0 TX output
#define HOST_RX_BIT         BV(5)       // SCI0 RX input
#define HOST_UART_BITS      (HOST_TX_BIT + HOST_RX_BIT)

// Port 3 Inits
#define P3SEL_INIT          (HOST_UART_BITS) // enable SCI0 UART
#define P3OUT_INIT          (0x00)      // start with all output pins LOW
#define P3DIR_INIT          (~HOST_RX_BIT) // set I/O direction bits

// Port 3 Access Macros
// (not used)
#endif

#ifdef __msp430_have_port4
// Port 4 Bits
// (not used)

// Port 4 Inits
#define P4SEL_INIT          (0x00)      // all pins are port pins
#define P4OUT_INIT          (0x00)      // start with all output pins LOW
#define P4DIR_INIT          (0xFF)      // all pins are outputs

// Port 4 Access Macros
// (not used)
#endif

#ifdef __msp430_have_port5
// Port 5 Bits
// (not used)

// Port 5 Inits
#define P5SEL_INIT          (0x00)      // all pins are port pins
#define P5OUT_INIT          (0x00)      // start with all output pins LOW
#define P5DIR_INIT          (0xFF)      // all pins are outputs

// Port 5 Access Macros
// (not used)
#endif

#ifdef __msp430_have_port6
// Port 6 Bits
// (not used)

// Port 6 Inits
#define P6SEL_INIT          (0x00)      // all pins are port pins
#define P6OUT_INIT          (0x00)      // start with all output pins LOW
#define P6DIR_INIT          (0xFF)      // all pins are outputs

// Port 6 Access Macros
// (not used)
#endif

#endif
