#include <stdlib.h>
#include <io.h>
#include "config.h"
#include "task.h"

#define TASK1_STACK_SIZE  0x20
#define TASK1_STACK_START (0 + TASK1_STACK_SIZE)
#define TASK2_STACK_SIZE  0x20
#define TASK2_STACK_START (TASK1_STACK_START + TASK2_STACK_SIZE)
#define TASK3_STACK_SIZE  0x20
#define TASK3_STACK_START (TASK2_STACK_START + TASK3_STACK_SIZE)
#define TASK_STACK_SIZE   TASK3_STACK_START

unsigned char t1count = 0;
unsigned char t2count = 0;
unsigned char t2overflow = 0;
unsigned char t3count = 0;
unsigned char count = 0;

void task1(void);
void task2(void);
void task3(void);


static void cpuInit(void)
{
  register int delay;

  WDTCTL = WDTPW + WDTHOLD;             // stop Watchdog
  BCSCTL1 |= XTS + DIVA1 + XT2OFF;      // LFXT1 = HF XTAL
                                        // ACLK = LFXT1/4
                                        // XT2 Osc is OFF

  // setup ports while waiting for (LF)XT1 to come up
#ifdef __msp430_have_port0
  // setup port 0 from values in config.h
  P0SEL = P0SEL_INIT;
  P0OUT = P0OUT_INIT;
  P0DIR = P0DIR_INIT;
#endif

#ifdef __msp430_have_port1
  // setup port 1 from values in config.h
  P1SEL = P1SEL_INIT;
  P1OUT = P1OUT_INIT;
  P1DIR = P1DIR_INIT;
#endif

#ifdef __msp430_have_port2
  // setup port 2 from values in config.h
  P2SEL = P2SEL_INIT;
  P2OUT = P2OUT_INIT;
  P2DIR = P2DIR_INIT;
#endif

#ifdef __msp430_have_port3
  // setup port 3 from values in config.h
  P3SEL = P3SEL_INIT;
  P3OUT = P3OUT_INIT;
  P3DIR = P3DIR_INIT;
#endif

#ifdef __msp430_have_port4
  // setup port 4 from values in config.h
  P4SEL = P4SEL_INIT;
  P4OUT = P4OUT_INIT;
  P4DIR = P4DIR_INIT;
#endif

#ifdef __msp430_have_port5
  // setup port 5 from values in config.h
  P5SEL = P5SEL_INIT;
  P5OUT = P5OUT_INIT;
  P5DIR = P5DIR_INIT;
#endif

#ifdef __msp430_have_port6
  // setup port 6 from values in config.h
  P6SEL = P6SEL_INIT;
  P6OUT = P6OUT_INIT;
  P6DIR = P6DIR_INIT;
#endif

  // wait for (LF)XT1 oscillator to come up
  // clear OSC fault flag & set delay count
  // exit loop when no fault for delay count
  for (IFG1 &= ~OFIFG, delay = 255; delay; )
    {
    if (IFG1 & OFIFG)                   // check OSC fault flag
      {
      // it's set
      IFG1 &= ~OFIFG;                   // clear OSC fault flag
      delay = 255;                      // reset delay count
      }
    else
      --delay;                          // clear, decrement delay
    }

  BCSCTL2 |= SELM1 + SELM0 +            // MCLK = LFXT1/1
             DIVS1 + DIVS0;             // SMCLK = DCOCLK/8

#ifdef RELEASE
  // LAST step is re-enable the watchdog timer
  // - use tSMCLK * 2e15 (approx 350mSec)
  WDTCTL = WDTPW + WDTCNTCL;
#endif
}

void task1(void)
{
  for (;;)
    {
    t1count++;
    swap();
    }
}


void task2(void)
{
  for (;;)
    {
    if (++t2count == 0)
      t2overflow++;

    swap();
    }
}


void task3(void)
{
  for (;;)
    {
    if (t3count++ == 5)
      break;

    swap();
    }

  // returning should implicitly call terminate(NULL)
}


int main(void)
{
  void *t0ID, *t1ID, *t2ID, *t3ID;
  void *stacks[TASK_STACK_SIZE];        // stack space for tasks 

  cpuInit();
  t0ID = initTasking();                 // initialize the tasker
  t1ID = fork(task1, &stacks[TASK1_STACK_START]); // register task 1
  t2ID = fork(task2, &stacks[TASK2_STACK_START]); // register task 2
  t3ID = fork(task3, &stacks[TASK3_STACK_START]); // register task 3

  while (count < 15)
    {
    if (t1ID && count >= 10)
      {
      terminate(t1ID);
      t1ID = NULL;
      }

    count++;
    swap();
    }

  terminate(NULL);
  return 0;
}
