/******************************************************************************
 *
 * $Workfile: task.h $
 *
 * This module provides the interface for a Co-operative Multi-tasker
 * Copyright 1996-1997,2000-2002, R O SoftWare
 *
 * No guarantees, warrantees, or promises, implied or otherwise.
 * May be used for hobby or commercial purposes provided copyright
 * notice remains intact.
 *
 * $History: $
 * 
 *****************************************************************************/
#ifndef INC_TASK_H
#define INC_TASK_H

// define number of tasks (including main())
#define MAX_TASKS 6


/******************************************************************************
 *
 * Function Name: initTasking()
 *
 * Description:
 *    This function initializes the tasker, setting up the calling
 *    function as a task using the current SP for the stack.
 *
 * Calling Sequence:
 *    void
 *
 * Returns:
 *    ID of the current task
 *
 *****************************************************************************/
void *initTasking(void);

/******************************************************************************
 *
 * Function Name: fork()
 *
 * Description:
 *    This function installs the referenced function as a task using
 *    the referenced stack.
 *
 * Calling Sequence:
 *    void (*f) - address of task function
 *    void **stackBase - pointer to base of the stack
 *
 * Returns:
 *    ID of created task
 *    NULL on error
 *
 *****************************************************************************/
void *fork(void (*f)(void), void **stackBase);

/******************************************************************************
 *
 * Function Name: getTaskID()
 *
 * Description:
 *    This function returns the current task ID
 *
 * Calling Sequence:
 *    void
 *
 * Returns:
 *    the current task ID
 *
 *****************************************************************************/
void *getTaskID(void);

/******************************************************************************
 *
 * Function Name: swap()
 *
 * Description:
 *    This function saves the current task state, services the watchdog
 *    timer, changes to the next task state, then returns.
 *
 * Calling Sequence:
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
void swap(void);

/******************************************************************************
 *
 * Function Name: terminate()
 *
 * Description:
 *    This function deletes the referenced task from the task list,
 *    changes to the next task state, then returns.
 *
 * Calling Sequence:
 *    *taskID - task to terminate, current task if NULL
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
void terminate(void *taskID);

#endif
