#include "hardware.h"
#include "signal.h"

.extern line
.extern state
.extern freq
.extern play
.extern sde
.extern videomem

.text

/**
minimal delay function give time in R4

4 cycles per count == 0.5us
*/
.global ndelay
.type   ndelay,@function
ndelay:
        nop                     ;1
        dec   R4                ;1
        jnz   ndelay            ;2
        ret


/**
Interrupt handler for timer (timer_a0).

Caputre / Compare interrupts from module 0
*/
.global ta0Int
//.global vector_fff2
//vector_fff2:
interrupt(TIMERA0_VECTOR)
ta0Int:
        add    state, R0
        jmp    L_A   ;621
        jmp    L_A   ;622
        jmp    L_B   ;623
        jmp    L_B   ;624
        jmp    L_B   ;625
        jmp    L_C   ;1
        jmp    L_C   ;2
        jmp    L_D   ;3
        jmp    L_B   ;4
        jmp    L_B   ;5
        jmp    L_A   ;6
        jmp    L_A   ;7
        jmp    L_A   ;8
        jmp    BLANKLINES
        jmp    LINES
        jmp    BLANKLINES
        jmp    L_A   ;309
        jmp    L_A   ;310
        jmp    L_B   ;311
        jmp    L_B   ;312
        jmp    L_E   ;313
        jmp    L_C   ;314
        jmp    L_C   ;315
        jmp    L_B   ;316
        jmp    L_B   ;317
        jmp    L_A   ;318
        jmp    L_A   ;319
        jmp    L_A   ;320
        jmp    BLANKLINES
        jmp    LINES
        jmp    BLANKLINES
        clr    state	;reset state and go into L_A
        

L_A:
        ;prepulse long high
        ;sync
        bic    #OUT, &CCTL0
        bic.b  #BIT0, &P1OUT
        mov    #4, R4
        call   #ndelay
        ;long high
        bis    #OUT, &CCTL0       
        mov    #100, R4
        call   #ndelay

        incd   state
        jmp    eol

L_B:
        ;prepulse long high with gap
        ;sync
        bic    #OUT, &CCTL0
        bic.b  #BIT0, &P1OUT
        mov    #4, R4
        call   #ndelay
        ;long high
        bis    #OUT, &CCTL0       
        mov    #52, R4
        call   #ndelay
        ;short gap
        bic    #OUT, &CCTL0       
        mov    #4, R4
        call   #ndelay
        ;long high
;        bis    #OUT, &CCTL0       
;        mov    #44, R4
;        call   #ndelay

        incd   state
        jmp    eol

L_C:
        ;prepulse long low with short pulse
        ;sync, lon low
        bic    #OUT, &CCTL0
        bic.b  #BIT0, &P1OUT
        mov    #43, R4
        call   #ndelay
        bis    #OUT, &CCTL0       
        ;short high
        bis    #OUT, &CCTL0
        mov    #6, R4
        call   #ndelay
        ;long low
        bic    #OUT, &CCTL0
        mov    #45, R4
        call   #ndelay
        ;short high
        bic    #OUT, &CCTL0
        mov    #6, R4
        call   #ndelay

        incd   state
        jmp    eol

L_D:
        ;prepulse long low with short pulse then high
        ;sync, long low
        bic    #OUT, &CCTL0
        bic.b  #BIT0, &P1OUT
        mov    #52, R4
        call   #ndelay
        ;pulse
        bis    #OUT, &CCTL0       
        mov    #4, R4
        call   #ndelay
        ;shortlow
        bic    #OUT, &CCTL0       
        mov    #4, R4
        call   #ndelay
        ;high to end
        bis    #OUT, &CCTL0
        mov    #40, R4
        call   #ndelay

        incd   state
        clr    line
        bic    #CPUOFF|OSCOFF|SCG0|SCG1, 0(R1)   ; set CPU to active mode, the alarm could have set a taskbit
        jmp    eol

L_E:
        ;prepulse long high with gap
        ;sync
        bic    #OUT, &CCTL0
        bic.b  #BIT0, &P1OUT
        mov    #4, R4
        call   #ndelay
        ;long high
        bis    #OUT, &CCTL0
        mov    #48, R4
        call   #ndelay
        ;long low
        bic    #OUT, &CCTL0
        mov    #50, R4
        call   #ndelay

        incd   state
        clr    line
        bic    #CPUOFF|OSCOFF|SCG0|SCG1, 0(R1)   ; set CPU to active mode, the alarm could have set a taskbit
        jmp    eol

BLANKLINES:
        ;start with sync pulse
        bic    #OUT, &CCTL0
        bic.b  #BIT0, &P1OUT
        
        mov    #7, R4
        call   #ndelay

        bis    #OUT, &CCTL0

        inc    line
        cmp    #54, line
        jne    eol
        incd   state
        clr    line
        jmp    eol


LINES:
        ;start with sync pulse
        bic    #OUT, &CCTL0	;4
        bic.b  #BIT0, &P1OUT	;3
        
        push   R9		;2
        push   R11		;2
        push   R12		;2

        push   r5       ;XXX
        push   r4       ;XXX

        mov    #16, R5		;2 number of bits to shift out
        mov    R5, R9		;1 and for the second register

        mov    line, R4		;3
        rra    R4		;1   div 2 = ram address (-->8 times the same data for a pixel)
        bic    #BIT0|BIT1, R4	;2   mask the lower four bytes
        mov    videomem(R4), R12;6
        incd   R4		;1
        mov    videomem(R4), R11;6
        
;        mov    #1, R4
;        call   #ndelay

        bis    #OUT, &CCTL0	;4 

        mov    #20, R4		;3
        call   #ndelay		;3


        ;line data        

ta0s1:  rlc     R11		;2
        rlc.b   &P1OUT		;2
        dec     R5		;1
        jnz     ta0s1		;2

        
ta0s2:  rlc     R12		;2
        rlc.b   &P1OUT		;2
        dec     R9		;1
        jnz     ta0s2		;2
        

        pop    r4       ;XXX
        pop    r5       ;XXX

        pop    R12		;2
        pop    R11		;2

        bic.b  #BIT0, &P1OUT

        pop    R9

        inc    line
        cmp    #192, line
        jne    eol
        incd   state       
        clr    line


        ;end of line
eol:    bic.b  #BIT0, &P1OUT
        bis    #OUT, &CCTL0
        
        ;audio output
        jmp    er       ;XXX DEBUG no sound
        tst    play		;playing ?
        jnz    pl
        clr    &CCTL1
        reti			;no -> exit

pl:     dec    play
        dec    sde
        jnz    er
        mov    freq, sde
        xor    #OUT, &CCTL1
er:     reti


interrupt (TIMERA1_VECTOR)
    //demux timer interrupts
    add     TAIV, r0                ; Add TA interrupt offset to PC
    reti                            ; CCR0 - no source
    jmp ccr1                        ; CCR1
    jmp ccr2                        ; CCR2
    reti                            ; CCR3 - no source
    reti                            ; CCR4 - no source
            br #INT_TimerA_TAOVER   ; TAOVER (follows directly)
    ccr1:   br #INT_TimerA_CCR1
    ccr2:   br #INT_TimerA_CCR2


/**
Interrupt handler for NMI / Oscillator fault.

Osc. fault is set when thy crystal oscilator is stopped or blocked for a moment. Its also
generated when the JTAG interface is used. Therefore this interrupt handler does reset the
oscillator fault bit.
This part of the interrupt handler is only needed if a crystal oscillator is used.
*/
interrupt (NMI_VECTOR)
nmiOsc:
fault:  bic.b #OFIFG, &IFG1             ; check oscilator fault bit
        bit.b #OFIFG, &IFG1             ; clear oscilator fault bit
        jc    fault                     ; test if its reset, otherwise try reset it again
        bis.b #OFIE, &IE1               ; re-enable IRQ, bit was reseted automacicaly
        reti


